--===================================================
-- Widget Options 
--===================================================
local defaultOptions  = {
	--Fuel Alarm (%)
	{"Fuel_Alarm", VALUE, 25, 1, 100 },
}

--===================================================
-- Get telemetry ID from name
-- There are the sensors detected by the Horus from the Telemetry module
-- Check them after detected and change the number by the name as follow
--	4400 = EGTC
-- 	4401 = RPM1
--	4402 = THRT
--	4403 = VBAT
--	4404 = PUMP
-- 	4405 = FUEL
--	4406 = STAT
-- 	4414 = RPM2
--	4415 = TEMP

--===================================================
local function getTelemetryId(name)
    field = getFieldInfo(name)
    if field then
      return field['id']
    else
      return -1
    end
end

--===================================================
-- This function returns green at 100%, red bellow Lowlv % set in widget settings and graduate in betwwen
--===================================================
local function getPercentColor(cpercent,lowlevel)
  if cpercent < lowlevel then
    return lcd.RGB(0xff, 0, 0)
  else
    g = math.floor(0xdf * cpercent / 100)
    r = 0xdf - g
    return lcd.RGB(r, g, 0)
  end
end

--===================================================
-- Rounding function
--local function round(num, numDecimalPlaces)
	--return tonumber(string.format("%." .. (numDecimalPlaces or 0) .. "f", num))
--end

--===================================================
-- Setup the status message table based on ECU type
--===================================================
local function initECU_Status(Me)
		Me.ECU_Status[0]  = "HighTemp"
		Me.ECU_Status[1]  = "Trim Low"
		Me.ECU_Status[2]  = "SetIdle!"
		Me.ECU_Status[3]  = "Ready"
		Me.ECU_Status[4]  = "Ignition"
		Me.ECU_Status[5]  = "FuelRamp"
		Me.ECU_Status[6]  = "Glow Test"
		Me.ECU_Status[7]  = "Running"
		Me.ECU_Status[8]  = "Stop"
		Me.ECU_Status[9]  = "FlameOut"
		Me.ECU_Status[10] = "SpeedLow"
		Me.ECU_Status[11] = "Cooling"
		Me.ECU_Status[12] = "Ignit.Bad"
		Me.ECU_Status[13] = "Start.Fail"
		Me.ECU_Status[14] = "AccelFail"
		Me.ECU_Status[15] = "Start On"
		Me.ECU_Status[16] = "UserOff"
		Me.ECU_Status[17] = "Failsafe"
		Me.ECU_Status[18] = "Low RPM"
		Me.ECU_Status[19] = "Reset"
		Me.ECU_Status[20] = "RXPwFail"
		Me.ECU_Status[21] = "PreHeat"
		Me.ECU_Status[22] = "Battery!"
		Me.ECU_Status[23] = "Time Out"
		Me.ECU_Status[24] = "Overload"
		Me.ECU_Status[25] = "Ign.Fail"
		Me.ECU_Status[26] = "Burner On"
		Me.ECU_Status[27] = "Starting"
		Me.ECU_Status[28] = "SwitchOv"
		Me.ECU_Status[29] = "Cal.Pump"
		Me.ECU_Status[30] = "PumpLimi"
		Me.ECU_Status[31] = "NoEngine"
		Me.ECU_Status[32] = "PwrBoost"
		Me.ECU_Status[33] = "Run-Idle"
		Me.ECU_Status[34] = "Run-Max "
		Me.ECU_Status[35] = "Restart "
		Me.ECU_Status[36] = "No Status"
end

--===================================================
-- Widget Creation
--===================================================
local function createWidget(zone, options)
	local imagePath = "/WIDGETS/TELEX/img"
	local Img_Logo = Bitmap.open(imagePath.."/Xicoy_logo.png")
	local Img_Chr = Bitmap.open(imagePath.."/chrono.png")
	local Img_Fuel = Bitmap.open(imagePath.."/fuel.png")
	local Img_Aircraft = Bitmap.open("/IMAGES/"..model.getInfo().bitmap)
	local Img_Aircraft_Path = model.getInfo().bitmap

	-- ECU Status table
	local ECU_Status = {}
	-- Initialize with number strings "0" to "100"
	for i=0, 100 do
		ECU_Status[i] = tostring(i)
	end
	
		
	local Me = { 
		zone=zone, options=options,
		imagePath=imagePath,
		Img_Logo = Img_Logo, Img_Aircraft = Img_Aircraft, Img_Aircraft_Path = Img_Aircraft_Path, Img_Chr = Img_Chr, Img_Fuel = Img_Fuel,
		ECU_Status = ECU_Status, Units = Units, param = {}
	}
	initECU_Status(Me)
	
	Me.param.Thro 	= getTelemetryId("THRT")
	Me.param.Pump   = getTelemetryId("PUMP")
	Me.param.Vbat   = getTelemetryId("VBAT")
	Me.param.Egt 	= getTelemetryId("EGTC")
	Me.param.Rpm1  	= getTelemetryId("RPM1")
	Me.param.Stat 	= getTelemetryId("STAT")
	Me.param.Rpm2	= getTelemetryId("RPM2")
	Me.param.Temp	= getTelemetryId("TEMP")
	Me.param.Fuel	= getTelemetryId("FUEL")
	
	return Me
end
--===================================================
-- Update the widget
--===================================================
local function updateWidget(Me, newOptions)
	Me.options = newOptions
end


--===================================================
-- Refresh the widget
--===================================================
local function refreshWidget(Me)
	--Save the original text color
	ORIG_COLOR = TEXT_COLOR
	
	--Status
	local status = getValue(Me.param.Stat)
	local rssi_value = getRSSI()
	if rssi_value == 0 then
		status = 36
	end
	local status_str = Me.ECU_Status[status] or tostring(status)
		
--==================================================
--WIDGET FULL SIZE
--==================================================
	if Me.zone.w > 380 and Me.zone.h > 165 then
		
	--Set the text color to black
		lcd.setColor(TEXT_COLOR,BLACK)
	
	--Logo image
		lcd.drawBitmap(Me.Img_Logo, Me.zone.x+190, Me.zone.y+50,10)
	--Aircraft image
		if Me.Img_Aircraft_Path ~= model.getInfo().bitmap then
			Me.Img_Aircraft_Path = model.getInfo().bitmap
			Me.Img_Aircraft = Bitmap.open("/IMAGES/"..model.getInfo().bitmap)
		end
		lcd.drawBitmap(Me.Img_Aircraft, Me.zone.x,Me.zone.y,70)	
	--Aircraft Name
		lcd.drawText(Me.zone.x,Me.zone.y+155,model.getInfo().name)
	--Timer
		lcd.drawBitmap(Me.Img_Chr,Me.zone.x+190,Me.zone.y+5,100)
		lcd.drawTimer(Me.zone.x+225,Me.zone.y-15,model.getTimer(0).value, XXLSIZE)
	--Turbine Info
		lcd.drawText(Me.zone.x+275,Me.zone.y+55,status_str,MIDSIZE+INVERS)
		lcd.drawText(Me.zone.x+220,Me.zone.y+90,"RPM:",SMLSIZE+INVERS)
		lcd.drawNumber(Me.zone.x+260,Me.zone.y+89,getValue(Me.param.Rpm1))
		lcd.drawText(Me.zone.x+220,Me.zone.y+112,"EGT(c):",SMLSIZE+INVERS)
		lcd.drawNumber(Me.zone.x+275,Me.zone.y+111,getValue(Me.param.Egt))
		lcd.drawText(Me.zone.x+220,Me.zone.y+132,"PW:",SMLSIZE+INVERS)
		lcd.drawNumber(Me.zone.x+255,Me.zone.y+131,getValue(Me.param.Pump))
		lcd.drawText(Me.zone.x+220,Me.zone.y+152,"ECU(v):",SMLSIZE+INVERS)
		lcd.drawNumber(Me.zone.x+275,Me.zone.y+151,getValue(Me.param.Vbat),PREC1)
		lcd.drawText(Me.zone.x+120,Me.zone.y+155,"THR(%):", SMLSIZE+INVERS)
		lcd.drawNumber(Me.zone.x+185,Me.zone.y+155,getValue(Me.param.Thro),SMLSIZE )
	--2nd Shaft Info
		if getValue(Me.param.Rpm2) > 0 then
			lcd.drawText(Me.zone.x+320,Me.zone.y+90," / ",SMLSIZE)
			lcd.drawNumber(Me.zone.x+335,Me.zone.y+89,getValue(Me.param.Rpm2))
			lcd.drawText(Me.zone.x+320,Me.zone.y+112," / ",SMLSIZE)
			lcd.drawNumber(Me.zone.x+335,Me.zone.y+111,getValue(Me.param.Temp))
		end
	--Fuel
		lcd.drawBitmap(Me.Img_Fuel,Me.zone.x,Me.zone.y+120)
		local Current_CUSTOM_COLOR = CUSTOM_COLOR
		lcd.setColor(CUSTOM_COLOR, getPercentColor(getValue(Me.param.Fuel),Me.options.Fuel_Alarm))
		lcd.drawGauge(Me.zone.x+30,Me.zone.y+120, 170, 30, getValue(Me.param.Fuel), 100,CUSTOM_COLOR)
		lcd.setColor(CUSTOM_COLOR,Current_CUSTOM_COLOR)
		lcd.drawText(Me.zone.x+100,Me.zone.y+115, getValue(Me.param.Fuel)..("%"), DBLSIZE)
		
--=================================================		
--WIDGET HALF SIZE
--=================================================			
	elseif Me.zone.w > 180 and Me.zone.h > 145 then
		
		--Set the text color to black
		lcd.setColor(TEXT_COLOR,BLACK)
		
		--Background image
		lcd.drawBitmap(Me.Img_Logo, Me.zone.x,Me.zone.y,10)
		--Turbine Info
		lcd.drawText(Me.zone.x+80,Me.zone.y,status_str,MIDSIZE+INVERS)
		lcd.drawText(Me.zone.x,Me.zone.y+32,"RPM :",SMLSIZE)
		lcd.drawNumber(Me.zone.x+55,Me.zone.y+30,getValue(Me.param.Rpm1))
		lcd.drawText(Me.zone.x,Me.zone.y+50,"EGT(c):",SMLSIZE)
		lcd.drawNumber(Me.zone.x+55,Me.zone.y+48,getValue(Me.param.Egt))
		lcd.drawText(Me.zone.x,Me.zone.y+68,"PW :",SMLSIZE)
		lcd.drawNumber(Me.zone.x+55,Me.zone.y+66,getValue(Me.param.Pump))
		lcd.drawText(Me.zone.x,Me.zone.y+86,"ECU(v):",SMLSIZE)
		lcd.drawNumber(Me.zone.x+55,Me.zone.y+84,getValue(Me.param.Vbat),PREC1)
		lcd.drawText(Me.zone.x,Me.zone.y+104,"THR(%):",SMLSIZE)
		lcd.drawNumber(Me.zone.x+55,Me.zone.y+104,getValue(Me.param.Thro),SMLSIZE)
		--2nd Shaft Info
		if getValue(Me.param.Rpm2) > 0 then
			lcd.drawText(Me.zone.x+120,Me.zone.y+32," / ",SMLSIZE)
			lcd.drawNumber(Me.zone.x+134,Me.zone.y+30,getValue(Me.param.Rpm2))
			lcd.drawText(Me.zone.x+120,Me.zone.y+50," / ",SMLSIZE)
			lcd.drawNumber(Me.zone.x+134,Me.zone.y+48,getValue(Me.param.Temp))
		end
		--Fuel
		local Current_CUSTOM_COLOR = CUSTOM_COLOR
		lcd.setColor(CUSTOM_COLOR, getPercentColor(getValue(Me.param.Fuel),Me.options.Fuel_Alarm))
		lcd.drawGauge(Me.zone.x+45,Me.zone.y+125, 150, 30, getValue(Me.param.Fuel), 100,CUSTOM_COLOR)
		lcd.setColor(CUSTOM_COLOR,Current_CUSTOM_COLOR)
		lcd.drawText(Me.zone.x,Me.zone.y+130,"FUEL :",SMLSIZE)
		lcd.drawText(Me.zone.x+110,Me.zone.y+120, getValue(Me.param.Fuel)..("%"), DBLSIZE)

--=================================================		
--WIDGET 1/8 SIZE just FUEL
--=================================================	
	elseif Me.zone.w == 160 and Me.zone.h == 32 then
	
		--Set the text color to black
		lcd.setColor(TEXT_COLOR,BLACK)
		--Fuel
		local Current_CUSTOM_COLOR = CUSTOM_COLOR
		lcd.setColor(CUSTOM_COLOR, getPercentColor(getValue(Me.param.Fuel),Me.options.Fuel_Alarm))
		lcd.drawGauge(Me.zone.x+5,Me.zone.y, 150, 30, getValue(Me.param.Fuel), 100,CUSTOM_COLOR)
		lcd.setColor(CUSTOM_COLOR,Current_CUSTOM_COLOR)
		lcd.drawText(Me.zone.x+75,Me.zone.y-5, getValue(Me.param.Fuel)..("%"), DBLSIZE)
	
--=================================================		
--WIDGET ERROR Due to wrong widget size
--=================================================		
	else
		--Set the text color to black
		lcd.setColor(TEXT_COLOR,BLACK)
		lcd.drawText(Me.zone.x+5,Me.zone.y+5,"ERROR",SMLSIZE)
		lcd.drawText(Me.zone.x+5,Me.zone.y+15,"WIDGET WINDOWS",SMLSIZE)
	
	end
	
	--set the text color back to the original
	lcd.setColor(TEXT_COLOR,ORIG_COLOR)
end

--===================================================
-- Background process (when widget not visible)
--===================================================
local function backgroundProcessWidget(Me)
	
end

--===================================================
-- Widget return functions
--===================================================
return { name="TELEX", options=defaultOptions, create=createWidget, update=updateWidget, background=backgroundProcessWidget, refresh=refreshWidget }